# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import
from unittest import mock, TestCase
from mojo.utils import bicommand, is_config_line, merge_dictionaries, secs2human


class UtilsTest(TestCase):
    def test_bicommand_environment_expansion(self):
        """Environment variables in bicommand invocations are expanded."""
        response = "I've come to talk with you again."
        status, output = bicommand(
            "echo ${HELLO_DARKNESS_MY_OLD_FRIEND}", env={"HELLO_DARKNESS_MY_OLD_FRIEND": response}
        )
        self.assertEqual(status, 0)
        self.assertEqual(output, response)

    @mock.patch('select.select')
    @mock.patch('subprocess.Popen')
    @mock.patch('sys.stdout')
    def test_bicommand_decoding(self, stdout, popen, select):
        def select_side_effect(rlist, unused_wlist, unused_xlist, unused_timeout):
            return rlist, None, None

        select.side_effect = select_side_effect

        stdout_write = mock.MagicMock()
        stdout.encoding = mock.MagicMock().return_value = 'utf-8'
        stdout.write = stdout_write

        response = b'Hello \xe2\x80\x93 World!'

        response_bytes = [b.to_bytes(1, byteorder='big') for b in response]
        response_bytes.append(b'') # EOF
        self.assertEqual(len(response_bytes), 17)

        popen_read = mock.MagicMock()
        popen_read.side_effect = response_bytes

        popen_stdout = mock.MagicMock()
        popen_stdout.fileno = mock.MagicMock().return_value = 1
        popen_stdout.read = popen_read

        bic_popen = popen()
        bic_popen.stdin = None
        bic_popen.stdout = popen_stdout
        bic_popen.stderr = None
        bic_popen.returncode = 0
        bic_popen.wait = mock.MagicMock().return_value = lambda: 0

        status, output = bicommand('dummy command', showoutput=True)
        self.assertEqual(status, 0)
        self.assertEqual(output, response.decode('utf-8'))

        calls = [mock.call('')]
        for c in response.decode('utf-8'):
            calls.append(mock.call(c))
        calls.append(mock.call(''))
        stdout_write.assert_has_calls(calls)

    def test_is_config_line(self):
        """Test various types of lines for whether they are valid config or not"""
        self.assertEqual(True, is_config_line("This is some config"))
        self.assertEqual(True, is_config_line(" This is some config "))
        self.assertEqual(False, is_config_line(" "))
        self.assertEqual(False, is_config_line("# This is not config"))
        self.assertEqual(False, is_config_line(" # This is not config"))

    def test_merge_dictionaries(self):
        dict_1 = {"applications": {"test-app": {"options": {"active": False}}}}
        dict_2 = {"applications": {"test-app": {"options": {"metrics_target": ""}}}}
        expected = {"applications": {"test-app": {"options": {"active": False, "metrics_target": ""}}}}
        self.assertEqual(merge_dictionaries(dict_1, dict_2), expected)

    def test_secs2human(self):
        self.assertEqual(secs2human(None), None)
        self.assertEqual(secs2human(False), None)
        self.assertEqual(secs2human(True), None)
        self.assertEqual(secs2human("some-string"), None)
        self.assertEqual(secs2human("671"), "11m 11s")
        self.assertEqual(secs2human(0), "0s")
        self.assertEqual(secs2human(0.01234567), "0s")
        self.assertEqual(secs2human(46.21), "46s")
        self.assertEqual(secs2human(46.61), "47s")
        self.assertEqual(secs2human(60), "1m")
        self.assertEqual(secs2human(71), "1m 11s")
        self.assertEqual(secs2human(600), "10m")
        self.assertEqual(secs2human(771), "12m 51s")
        self.assertEqual(secs2human(3600), "1h")
        self.assertEqual(secs2human(3660), "1h 1m")
        self.assertEqual(secs2human(7771), "2h 9m 31s")
        self.assertEqual(secs2human(86400), "24h")
        self.assertEqual(secs2human(87000), "24h 10m")
        self.assertEqual(secs2human(90061), "25h 1m 1s")
